# Generated by Django 4.0.4 on 2022-05-11 12:01

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re


class Migration(migrations.Migration):

    dependencies = [
        ('labeltree', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='label',
            unique_together={('name', 'taxonomic_rank')},
        ),
        migrations.AlterField(
            model_name='label',
            name='kr_nr',
            field=models.CharField(blank=True, max_length=255, null=True, validators=[django.core.validators.RegexValidator(message='K+R numbers should be alpha-numeric only!', regex=re.compile('[A-Za-z0-9]+'))], verbose_name='K+R-Nr.'),
        ),
        migrations.AlterField(
            model_name='label',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', related_query_name='child', to='labeltree.label', verbose_name='Parent'),
        ),
        migrations.AlterUniqueTogether(
            name='label',
            unique_together={('name', 'kr_nr'), ('name', 'taxonomic_rank')},
        ),
    ]
