from django.test import TestCase

# Create your tests here.

class ListLabels(TestCase):

    def test_something(self):
        self.assertEqual(2, 4)

    def test_something_else(self):
        self.assertEqual(4, 4)
