# Generated by Django 4.0.4 on 2022-05-15 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labeltree', '0003_labelgroup'),
    ]

    operations = [
        migrations.AlterField(
            model_name='label',
            name='taxonomic_rank',
            field=models.CharField(choices=[('order', 'Order'), ('family', 'Family'), ('genus', 'Genus'), ('species', 'Species')], default='species', max_length=255, verbose_name='Taxonomic Rank'),
        ),
    ]
